
//@module : Dummy_Cores_Mesh_Wrapper
//@author : heracles-gui

/* 
 *  Using Heracles Multicore System under the copytright below:
 *  Copyright (c) 2010 Heracles (CSG/CSAIL/MIT)
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.

 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 *
 */

module Dummy_Cores_Mesh_Wrapper (
	input  clock,
    input  RST,	
	input  cmd,   
	output [31: 0] data, 
	output valid
);  

//define the log2 function
function integer log2;
	input integer num;
	integer i, result;
	begin
		for (i = 0; 2 ** i < num; i = i + 1)
			result = i + 1;
		log2 = result;
	end
endfunction

//----------------------------------------------------
// Parameters
//----------------------------------------------------
localparam  VC_PER_PORTS    = 2;
localparam  VC_DEPTH        = 8;
localparam  COLUMN          = 2;
localparam  EXTRA           = 2;
localparam  DATA_WIDTH      = 32;
localparam  ROW             = 2;
localparam  MSG_BITS        = 0;
localparam  SWITCH_TO_SWITCH= 1;
localparam  RT_ALG          = 0;
localparam  STATS_CYCLES    = 100000;
localparam  TYPE_BITS       = 2;
localparam  OUT_PORTS       = 1;
localparam  RATE            = 10;
localparam  FLITS_PER_PACKET= 1;
localparam  SIM_CYCLES      = 100000;
localparam  TRAFFIC         = 2;

localparam  ROW_BITS     = log2(ROW);
localparam  COLUMN_BITS  = log2(COLUMN);
localparam  VC_BITS      = log2(VC_PER_PORTS);		 
localparam  ID_BITS      = ROW_BITS + COLUMN_BITS;					 
localparam  FLOW_BITS    = (2*ID_BITS) + EXTRA;  
localparam  PORTS        = OUT_PORTS + 4*(SWITCH_TO_SWITCH);
localparam  PORTS_BITS   = log2(PORTS); 
localparam  RT_WIDTH     = PORTS_BITS + VC_BITS + 1;

//----------------------------------------------------
// Unit under test inputs and outputs
//----------------------------------------------------    
reg [31:0]               prog_address;	
reg [3: 0]               operation;   
reg                      reset;
reg                      start;
reg                      stop;
reg                      PROG; 
reg                      ON;
reg [ID_BITS-1: 0]       core_ID; 
reg [ID_BITS-1: 0]       dest_ID; 
reg [(FLOW_BITS - 1): 0] route_table_address; 
reg [RT_WIDTH - 1: 0]    route_table_data; 
wire[ID_BITS-1: 0]       origin;
wire[31: 0]              data_out; 
wire                     valid_out;

reg [ID_BITS-1: 0]       turn; 
reg [31:0]               cycles; 
    
//----------------------------------------------------
// Connect the DUT
//----------------------------------------------------
Dummy_Cores_Mesh #(.ROW(ROW), .COLUMN(COLUMN),
				   .VC_PER_PORTS(VC_PER_PORTS),.VC_DEPTH(VC_DEPTH),
				   .RT_ALG(RT_ALG), .EXTRA(EXTRA), .DATA_WIDTH(DATA_WIDTH), 
				   .TRAFFIC(TRAFFIC), .RATE(RATE), 
				   .FLITS_PER_PACKET(FLITS_PER_PACKET), 
				   .STATS_CYCLES(STATS_CYCLES)) U (
  clock, 
  RST, 
  start, 
  stop, 
  
  operation, 
  ON, 
  core_ID, 
  dest_ID, 
  reset, 
  PROG,
  
  route_table_address, 
  route_table_data, 
  origin, 
  data_out, 
  valid_out 
);  

//----------------------------------------------------
// Output buffering 
//----------------------------------------------------
wire          full; 
wire          empty; 
wire          read_valid;
wire [31: 0]  read_data; 
reg           rdEn;
reg  [4:0]    state;

wire peek                = 1'b0; 
wire mesh_valid          = valid_out && ~full;
wire [31: 0] mesh_data   = data_out;   

fifo  #(32, 8, 0) buffer (
			clock, RST, mesh_data, 
			mesh_valid, rdEn, peek, 
			read_data, read_valid, full, empty 
);  
		
//--------------Code Starts Here----------------------- 
always @ (posedge clock) begin
	if(~RST && cmd && ~empty && ~rdEn) begin 
		rdEn <= 1; 
	end 
	else rdEn <= 0;
end

always @ (posedge clock) begin 
  if (RST == 1) begin 
     cycles  <= 0; 
	 turn    <= 0;
     state   <= 0;   
	 reset 	 <= 0;
     PROG    <= 0; 
     ON      <= 0; 
     core_ID <= 0;
	 dest_ID <= 0;	 
	 start   <= 0;	 
	 stop    <= 0;	 
  end 
  else  begin
	  cycles  <= cycles + 1; 
      case (state)
			0: begin
			   state   <= state + 1;
			end		
			1: begin
				operation             <= 4'b0011;
				ON                    <= 1;
				reset                 <= 1;
				core_ID               <= 0;
				state   <= state + 1;
			end
			2: begin
				operation             <= 4'b0011;
				ON                    <= 1;
				reset                 <= 1;
				core_ID               <= 1;
				state   <= state + 1;
			end
			3: begin
				operation             <= 4'b0011;
				ON                    <= 1;
				reset                 <= 1;
				core_ID               <= 2;
				state   <= state + 1;
			end
			4: begin
				operation             <= 4'b0011;
				ON                    <= 1;
				reset                 <= 1;
				core_ID               <= 3;
				state   <= state + 1;
			end
			5: begin
				operation             <= 4'b1010;
				reset                 <= 0;
				PROG                  <= 0;
				core_ID               <= 0;
				dest_ID               <= 0;
				start                 <= 1;
				state   <= state + 1;
			end
			6: begin
				operation             <= 4'b1010;
				reset                 <= 0;
				PROG                  <= 0;
				core_ID               <= 1;
				dest_ID               <= 1;
				start                 <= 1;
				state   <= state + 1;
			end
			7: begin
				operation             <= 4'b1010;
				reset                 <= 0;
				PROG                  <= 0;
				core_ID               <= 2;
				dest_ID               <= 2;
				start                 <= 1;
				state   <= state + 1;
			end
			8: begin
				operation             <= 4'b1010;
				reset                 <= 0;
				PROG                  <= 0;
				core_ID               <= 3;
				dest_ID               <= 3;
				start                 <= 1;
				state   <= state + 1;
			end
			9: begin
				operation             <= 4'b0000;
				state   <= state + 1;
			end
			10: begin
				if(cycles > SIM_CYCLES) begin
					operation             <= 4'b1000;
					core_ID               <= turn; 
					start                 <= 0;
					stop                  <= 1;
					turn                  <= turn + 1;
				end
				state   <= state;
			end

			default:  begin
			   state    <= state; 
			end
		endcase 
	end
end
	
//----------------------------------------------------
// Drive the outputs
//----------------------------------------------------
 assign data  = read_data;
 assign valid = read_valid;
 
endmodule
        